/*
 * KIBA plugin for PIDGIN
 * Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 * Copyright from other people that wrote code with another purpose:
 * Copyright (C) 2007 Michael (mycroes) Croes <mycroes@gmail.com>
 * Copyright (C) 2002-3 Robert McQueen <robot101@debian.org>
 * Copyright (C) 2003 Herman Bloggs <hermanator12002@yahoo.com>
 *
 * Inspired by a similar plugin by:
 * Michael (mycroes) Croes <mycroes@gmail.com>
 * Which was in turn inspired by a similar plugin by:
 *   Robert McQueen <robot101@debian.org>
 *   Herman Bloggs <hermanator12002@yahoo.com>
 *   Which was in turn inspired by a similar plugin by:
 *    John (J5) Palmieri <johnp@martianrock.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

/* TODO:
- On Button 1 release:
	- no pending conversation:
		- gaim unmapped:
			map window and focus it
		- gaim mapped+unfocussed:
			focus gaim
		- gaim mapped+focussed:
			unmap
	- pending conversations
		- conversation window unmapped:
			map window, focus it and open conversation
		- conversation window mapped+unfocussed:
			focus conversation window and open conversation
		- conversation window mapped+focussed: 
			open conversation
- Titel
	Display a List of Online Buddies
- Translation doesnt work (luckily most translations are provided by gaim ;))
*/

#include <stdlib.h>
#include <libintl.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <glib.h>
#include <dbus/dbus.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-lowlevel.h>

#include "gaim-compat.h"
#include "gtkgaim-compat.h"
#include "gtkconv.h"

#include "core.h"
#include "debug.h"
#include "prefs.h"
#include "savedstatuses.h"
#include "version.h"
#include "plugin.h"

#include "gtkaccount.h"
#include "gtkblist.h"
#include "gtkplugin.h"
#include "gtkprefs.h"
#include "gtksavedstatuses.h"
#include "gtkstatusbox.h"
#include "gtkutils.h"
#include "gtkdialogs.h"

#include "pidgin.h"
#include "pidginstock.h"

#include "config.h"

#define PATH_IMG_OFFLINE         PIDGIN_PIXMAPDIR"/tray/48/tray-offline.png"
#define PATH_IMG_ONLINE          PIDGIN_PIXMAPDIR"/tray/48/tray-online.png"
#define PATH_IMG_ONLINE_PENDING  PIDGIN_PIXMAPDIR"/tray/48/tray-online.png"
#define PATH_IMG_AWAY            PIDGIN_PIXMAPDIR"/tray/48/tray-away.png"
#define PATH_IMG_AWAY_PENDING    PIDGIN_PIXMAPDIR"/tray/48/tray-away.png"
#define PATH_IMG_CONNECTING      PIDGIN_PIXMAPDIR"/tray/48/tray-connecting.png"
#define PATH_IMG_UNREAD          PIDGIN_PIXMAPDIR"/tray/48/tray-new-im.png"

#define _(String)             dgettext (GETTEXT_PACKAGE, String)

#define KIBA_PLUGIN_ID        "Kiba-Dock Pidgin Icon"
#define KIBA_PLUGIN_NAME      "pidgin-kiba"
#define KIBA_DBUS_PATH        "/org/kiba/dock/Kiba"
#define KIBA_NAMESPACE        "org.kiba.dock.Kiba"

typedef enum {
	KIBA_STATUS_OFFLINE = 0,
	KIBA_STATUS_ONLINE,
	KIBA_STATUS_ONLINE_PENDING,
	KIBA_STATUS_AWAY,
	KIBA_STATUS_AWAY_PENDING,
	KIBA_STATUS_CONNECTING
} KibaStatus;

/* keep the same order as KibaStatus enum above */
static const gchar* gaim_icon_data[] = {
	PATH_IMG_OFFLINE,         /**< KIBA_STATUS_OFFLINE.       */
	PATH_IMG_ONLINE,          /**< KIBA_STATUS_ONLINE.        */
	PATH_IMG_ONLINE_PENDING,  /**< KIBA_STATUS_ONLINE_PENDING.*/
	PATH_IMG_AWAY,            /**< KIBA_STATUS_AWAY.          */
	PATH_IMG_AWAY_PENDING,    /**< KIBA_STATUS_AWAY_PENDING.  */
	PATH_IMG_CONNECTING       /**< KIBA_STATUS_CONNECTING.    */
};

static KibaStatus status = KIBA_STATUS_OFFLINE;

static gboolean plugin_load(GaimPlugin *plugin);
static gboolean plugin_unload(GaimPlugin *plugin);

static gboolean kiba_update_status();

static gboolean online_account_supports_chat = FALSE;

static guint kiba_blinking_timer = 0;

/****************************
 * DBus methoid calls
 ****************************/

enum {
	KIBA_DBUS_PULSE = 0,
	KIBA_DBUS_ROTATE,
	KIBA_DBUS_NEW_ICON,
	KIBA_DBUS_SET_ICON,
	KIBA_DBUS_REMOVE_ICON,
	KIBA_DBUS_TITLE,
	KIBA_DBUS_SET_BUBBLE,
	KIBA_DBUS_UNSET_BUBBLE
};

static void call_dbus_method (gint     method,
			      gpointer data) {
	DBusGConnection *connection;
	DBusGProxy *proxy;
	GError *error;
	
	g_type_init();
	
	error = NULL;
	connection = dbus_g_bus_get(DBUS_BUS_SESSION, &error);
	
	if (connection == NULL)
		return;
	
	proxy = dbus_g_proxy_new_for_name(
			connection,
			KIBA_NAMESPACE,
			KIBA_DBUS_PATH,
			KIBA_NAMESPACE);
		
	switch (method) {
		case KIBA_DBUS_NEW_ICON:
			error = NULL;
			dbus_g_proxy_call(proxy, "AddObject", &error,
					G_TYPE_STRING, KIBA_PLUGIN_NAME,
					G_TYPE_STRING, "pidgin",
					G_TYPE_STRING, "pidgin", G_TYPE_INVALID);
			break;
		case KIBA_DBUS_REMOVE_ICON:
			error = NULL;
			dbus_g_proxy_call(proxy, "RemoveObject", &error,
					G_TYPE_STRING, KIBA_PLUGIN_NAME, G_TYPE_INVALID);
			break;
		case KIBA_DBUS_SET_ICON:
			error = NULL;
			dbus_g_proxy_call(proxy, "SetIcon", &error,
					G_TYPE_STRING, KIBA_PLUGIN_NAME,
					G_TYPE_STRING, (gchar*) data, G_TYPE_INVALID);
			break;
		case KIBA_DBUS_PULSE:
			error = NULL;
			dbus_g_proxy_call(proxy, "Pulse", &error, G_TYPE_STRING,
					KIBA_PLUGIN_NAME, G_TYPE_INVALID);
			break;
		case KIBA_DBUS_ROTATE:
			error = NULL;
			dbus_g_proxy_call(proxy, "Rotate", &error,
					G_TYPE_STRING, KIBA_PLUGIN_NAME, G_TYPE_INVALID);
			break;
		case KIBA_DBUS_TITLE:
			error = NULL;
			dbus_g_proxy_call(proxy, "SetTitel", &error,
					G_TYPE_STRING, KIBA_PLUGIN_NAME,
					G_TYPE_STRING, (gchar*) data, G_TYPE_INVALID);
			break;
		case KIBA_DBUS_SET_BUBBLE:
			error = NULL;
			dbus_g_proxy_call(proxy, "BubbleSetMessage", &error,
					G_TYPE_STRING, KIBA_PLUGIN_NAME,
					G_TYPE_STRING, (gchar*) data, G_TYPE_INVALID);
			break;
		case KIBA_DBUS_UNSET_BUBBLE:
			error = NULL;
			dbus_g_proxy_call(proxy, "BubbleUnsetMessage", &error,
					G_TYPE_STRING, KIBA_PLUGIN_NAME, G_TYPE_INVALID);
			break;
	}
}

/****************************
 * helper functions
 ****************************/

static void kiba_icon_set_bool(GtkWidget  *widget,
			       const char *key) {
	gaim_prefs_set_bool(key, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)));
}

static void update_icon(KibaStatus newStatus) {
	call_dbus_method (KIBA_DBUS_SET_ICON, (char*) gaim_icon_data[newStatus]);
}

static GList * get_pending_list(guint max) {
	GList *l_im = NULL;
	GList *l_chat = NULL;
	
	l_im = gaim_gtk_conversations_find_unseen_list(GAIM_CONV_TYPE_IM,
			GAIM_UNSEEN_TEXT, FALSE, max);
	
	l_chat = gaim_gtk_conversations_find_unseen_list(GAIM_CONV_TYPE_CHAT,
			GAIM_UNSEEN_NICK, FALSE, max);
	
	if (l_im != NULL && l_chat != NULL)
		return g_list_concat(l_im, l_chat);
	else if (l_im != NULL)
		return l_im;
	else
		return l_chat;
}

/**************************************************************************
 * kiba_icon pop-up menu
 **************************************************************************/

static void kiba_icon_toggle_mute(GtkWidget *toggle, void *data) {
	gaim_prefs_set_bool("/gaim/gtk/sound/mute", GTK_CHECK_MENU_ITEM(toggle)->active);
}

static void kiba_icon_toggle_blist(GtkWidget *toggle, void *data) {
	gaim_blist_set_visible(GTK_CHECK_MENU_ITEM(toggle)->active);
}

static void show_custom_status_editor_cb(GtkMenuItem *menuitem, gpointer user_data) {
	GaimSavedStatus *saved_status;
	saved_status = gaim_savedstatus_get_current();
	gaim_gtk_status_editor_show(FALSE,
		gaim_savedstatus_is_transient(saved_status) ? saved_status : NULL);
}

static void activate_status_primitive_cb(GtkMenuItem *menuitem, gpointer user_data) {
	GaimStatusPrimitive primitive;
	GaimSavedStatus *saved_status;

	primitive = GPOINTER_TO_INT(user_data);

	/* Try to lookup an already existing transient saved status */
	saved_status = gaim_savedstatus_find_transient_by_type_and_message(primitive, NULL);

	/* Create a new transient saved status if we weren't able to find one */
	if (saved_status == NULL)
		saved_status = gaim_savedstatus_new(NULL, primitive);

	/* Set the status for each account */
	gaim_savedstatus_activate(saved_status);
}

static void activate_saved_status_cb(GtkMenuItem *menuitem, gpointer user_data) {
	time_t creation_time;
	GaimSavedStatus *saved_status;

	creation_time = GPOINTER_TO_INT(user_data);
	saved_status = gaim_savedstatus_find_by_creation_time(creation_time);
	if (saved_status != NULL)
		gaim_savedstatus_activate(saved_status);
}

static GtkWidget* new_menu_item_with_gaim_icon(GtkWidget *menu, const char *str, GaimStatusPrimitive primitive, GtkSignalFunc sf, gpointer data, guint accel_key, guint accel_mods, char *mod) {
	GtkWidget *menuitem;
	GdkPixbuf *pixbuf;
	GtkWidget *image;

	menuitem = gtk_image_menu_item_new_with_mnemonic(str);

	if (menu)
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

	if (sf)
		g_signal_connect(G_OBJECT(menuitem), "activate", sf, data);

// 	pixbuf = gaim_gtk_create_gaim_icon_with_status(primitive, 0.5);
// 	image = gtk_image_new_from_pixbuf(pixbuf);
// 	g_object_unref(pixbuf);
// 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);

	gtk_widget_show_all(menuitem);

	return menuitem;
}

static GtkWidget* kiba_icon_status_submenu() {
	GtkWidget *submenu, *menuitem;
	GList *popular_statuses, *cur;

	submenu = gtk_menu_new();
	menuitem = gtk_menu_item_new_with_label(_("Change Status"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);

	new_menu_item_with_gaim_icon(submenu, _("Available"),
		GAIM_STATUS_AVAILABLE, G_CALLBACK(activate_status_primitive_cb),
		GINT_TO_POINTER(GAIM_STATUS_AVAILABLE), 0, 0, NULL);

	new_menu_item_with_gaim_icon(submenu, _("Away"),
		GAIM_STATUS_AWAY, G_CALLBACK(activate_status_primitive_cb),
		GINT_TO_POINTER(GAIM_STATUS_AWAY), 0, 0, NULL);

	new_menu_item_with_gaim_icon(submenu, _("Invisible"),
		GAIM_STATUS_INVISIBLE, G_CALLBACK(activate_status_primitive_cb),
		GINT_TO_POINTER(GAIM_STATUS_INVISIBLE), 0, 0, NULL);

	new_menu_item_with_gaim_icon(submenu, _("Offline"),
		GAIM_STATUS_OFFLINE, G_CALLBACK(activate_status_primitive_cb),
		GINT_TO_POINTER(GAIM_STATUS_OFFLINE), 0, 0, NULL);

	popular_statuses = gaim_savedstatuses_get_popular(6);
	if (popular_statuses != NULL)
		gaim_separator(submenu);
	for (cur = popular_statuses; cur != NULL; cur = cur->next)
	{
		GaimSavedStatus *saved_status = cur->data;
		time_t creation_time = gaim_savedstatus_get_creation_time(saved_status);
		new_menu_item_with_gaim_icon(submenu,
			gaim_savedstatus_get_title(saved_status),
			gaim_savedstatus_get_type(saved_status), G_CALLBACK(activate_saved_status_cb),
			GINT_TO_POINTER(creation_time), 0, 0, NULL);
	}
	g_list_free(popular_statuses);

	gaim_separator(submenu);

	new_menu_item_with_gaim_icon(submenu, _("New..."), GAIM_STATUS_AVAILABLE,
		 G_CALLBACK(show_custom_status_editor_cb), NULL, 0, 0, NULL);
	new_menu_item_with_gaim_icon(submenu, _("Saved..."), GAIM_STATUS_AVAILABLE,
		 G_CALLBACK(gaim_gtk_status_window_show), NULL, 0, 0, NULL);

	return menuitem;
}

static void createContextMenu() {
	static GtkWidget *menu = NULL;
	GtkWidget *menuitem;
	
	if (menu) {
		gtk_widget_destroy(menu);
	}
	
	menu = gtk_menu_new();
	
	menuitem = gtk_check_menu_item_new_with_label(_("Show Buddy List"));
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), gaim_prefs_get_bool("/gaim/gtk/blist/list_visible"));
	g_signal_connect(G_OBJECT(menuitem), "toggled", G_CALLBACK(kiba_icon_toggle_blist), NULL);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	
	menuitem = gtk_menu_item_new_with_label(_("Unread Messages"));
	
	if (status == KIBA_STATUS_ONLINE_PENDING || status == KIBA_STATUS_AWAY_PENDING) {
		GtkWidget *submenu = gtk_menu_new();
		GList *l = get_pending_list(0);
		if (l == NULL) {
			gtk_widget_set_sensitive(menuitem, FALSE);
			gaim_debug_warning("kiba_icon",
				"status indicates messages pending, but no conversations with unseen messages were found.");
		} else {
			gaim_gtk_conversations_fill_menu(submenu, l);
			g_list_free(l);
			gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);
		}
	} else {
		gtk_widget_set_sensitive(menuitem, FALSE);
	}
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	
	gaim_separator(menu);
	
	menuitem = gaim_new_item_from_stock(menu, _("New Message..."), PIDGIN_STOCK_TOOLBAR_MESSAGE_NEW,
		G_CALLBACK(gaim_gtkdialogs_im), NULL, 0, 0, NULL);
	if (status == KIBA_STATUS_OFFLINE)
		gtk_widget_set_sensitive(menuitem, FALSE);

	menuitem = kiba_icon_status_submenu();
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	
	gaim_separator(menu);
	
	gaim_new_item_from_stock(menu, _("Accounts"), PIDGIN_STOCK_TOOLBAR_ACCOUNTS,
		G_CALLBACK(gaim_gtk_accounts_window_show), NULL, 0, 0, NULL);
	gaim_new_item_from_stock(menu, _("Plugins"), PIDGIN_STOCK_TOOLBAR_PLUGINS,
		G_CALLBACK(gaim_gtk_plugin_dialog_show), NULL, 0, 0, NULL);
	gaim_new_item_from_stock(menu, _("Preferences"), GTK_STOCK_PREFERENCES,
		G_CALLBACK(gaim_gtk_prefs_show), NULL, 0, 0, NULL);
	
	gaim_separator(menu);
	
	gaim_separator(menu);
	
	switch (status) {
		case KIBA_STATUS_OFFLINE:
			break;
		default:
			gaim_new_item_from_stock(menu, _("Signoff"), GTK_STOCK_CLOSE,
				 G_CALLBACK(gaim_connections_disconnect_all), NULL, 0, 0, 0);
			break;
	}
	
	/* TODO: need a submenu to change status, this needs to "link"
	 * to the status in the buddy list gtkstatusbox
	 */
	
	gaim_new_item_from_stock(menu, _("Quit Pidgin"), GTK_STOCK_QUIT, G_CALLBACK(gaim_core_quit), NULL, 0, 0, NULL);
	
	gtk_widget_show_all(menu);
	
	gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
		       NULL,
		       NULL, 0, gtk_get_current_event_time());
}

/****************************
 * Kiba-Dock DBus communication
 ****************************/

static gint showMenu () {
	createContextMenu();
	return FALSE;
}

const char* kiba_get_message_string (DBusMessage *message) {
	DBusError error;
	gchar    *string;
	
	dbus_error_init (&error);
	if (!dbus_message_get_args (message,
				    &error,
				    DBUS_TYPE_STRING, &string,
				    DBUS_TYPE_INVALID)) {
		fprintf (stderr, "Signal received, but error getting message: %s\n",
				error.message);
		dbus_error_free (&error);
		return NULL;
	}
	
	return string;
}

static Atom gaim_get_atom (Display    *display,
			   const char *atom_name) {
	Atom retval;
	GHashTable *atom_hash = NULL, *reverse_atom_hash = NULL;
	g_return_val_if_fail (atom_name != NULL, None);
	
	atom_hash = g_hash_table_new (g_str_hash, g_str_equal);
	reverse_atom_hash = g_hash_table_new (NULL, NULL);
	
	retval = GPOINTER_TO_UINT (g_hash_table_lookup (atom_hash, atom_name));
	if (!retval) {
		retval = XInternAtom (display, atom_name, FALSE);
		
		if (retval != None) {
			char *name_copy;
			
			name_copy = g_strdup (atom_name);
			
			g_hash_table_insert (atom_hash, name_copy, GUINT_TO_POINTER (retval));
			g_hash_table_insert (reverse_atom_hash, GUINT_TO_POINTER (retval), name_copy);
			
			g_free (name_copy);
		}
	}
	
	g_hash_table_unref (atom_hash);
	g_hash_table_unref (reverse_atom_hash);
	
	return retval;
}

static DBusHandlerResult kiba_signal_filter (DBusConnection *dbus,
					     DBusMessage    *message,
					     void           *user_data) {
	DBusGConnection *connection = user_data;
	
	if (dbus_message_is_signal (message, KIBA_NAMESPACE, "IconMenu")) {
		const char *id = kiba_get_message_string (message);
		
		if (!id || strcmp (id, KIBA_PLUGIN_NAME) != 0)
			return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
		
		g_timeout_add(250, showMenu, NULL);
		
		return DBUS_HANDLER_RESULT_HANDLED;
	}
	else if (dbus_message_is_signal (message, KIBA_NAMESPACE, "IconReleased")) {
		const char *id = kiba_get_message_string (message);
		
		if (!id || strcmp (id, KIBA_PLUGIN_NAME) != 0)
			return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
		
		if (status == KIBA_STATUS_ONLINE_PENDING || status == KIBA_STATUS_AWAY_PENDING) {
			GList *l = get_pending_list(1);
			if (l != NULL) {
				gaim_gtkconv_present_conversation((GaimConversation *)l->data);
				g_list_free(l);
			}
		} else {
			//FIXME: there seems to be a bug with this function when the tray is deactivated
			gaim_gtk_blist_toggle_visibility();
		}
		
		return DBUS_HANDLER_RESULT_HANDLED;
	}
	/* Kiba-Dock was started after gaim */
	else if (dbus_message_is_signal (message, KIBA_NAMESPACE, "Initialled")) {
		
		call_dbus_method (KIBA_DBUS_NEW_ICON, NULL);
		status = KIBA_STATUS_OFFLINE;
		kiba_update_status();
		
		return DBUS_HANDLER_RESULT_HANDLED;
	}
	
	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
}

static void listenKibaSignals (DBusGConnection *connection) {
	DBusConnection *bus;
	DBusError       error;

	bus = dbus_g_connection_get_connection (connection);

	dbus_error_init (&error);
	/* listening to messages from all objects as no path is specified */
	dbus_bus_add_match (bus, "type='signal',interface='org.kiba.dock.Kiba'", &error);
	dbus_connection_add_filter (bus, kiba_signal_filter, connection, NULL);
}

/****************************
 * the real work
 ****************************/

static gboolean kiba_blink_icon() {
	static gboolean blinked = FALSE;
	gboolean ret = FALSE; /* by default, don't keep blinking */
	
	blinked = !blinked;
	
	switch (status) {
		case KIBA_STATUS_ONLINE_PENDING:
		case KIBA_STATUS_AWAY_PENDING:
			if (blinked) {
				update_icon(status - 1);
			} else {
				update_icon(status);
			}
			ret = TRUE; /* keep blinking */
			break;
		default:
			kiba_blinking_timer = 0;
			blinked = FALSE;
			break;
	}
	
	return ret;
}

static gboolean kiba_update_status() {
	GList *convs, *l, *c;
	int count;
	KibaStatus newstatus = KIBA_STATUS_OFFLINE;
	gboolean pending = FALSE;
	
	/* unseen messages */ /* no need to limit for kiba :) */
	convs = get_pending_list(0); /* KIBA_TOOLTIP_LINE_LIMIT */
	
	
	if (convs != NULL) {
		void *data = g_list_nth_data (convs, g_list_length (convs) - 1);
		
		pending = TRUE;
		
		if (GAIM_IS_GTK_CONVERSATION(data)) {
		char *message;
			
			GaimGtkConversation *gtkconv =
				GAIM_GTK_CONVERSATION((GaimConversation *) data);
			
				message = g_strdup_printf ("New message from %s", gtkconv->active_conv->title);
			
			if (gaim_prefs_get_bool("/plugins/gtk/kiba/show_bubble_info"))
				call_dbus_method (KIBA_DBUS_SET_BUBBLE, message);
			g_free (message);
		}
	}else {
		call_dbus_method (KIBA_DBUS_UNSET_BUBBLE, NULL);
	}
	
	online_account_supports_chat = FALSE;
	
	for (l = gaim_accounts_get_all(); l != NULL; l = l->next) {
		KibaStatus tmpstatus = KIBA_STATUS_OFFLINE;
		
		GaimAccount *account = (GaimAccount *)l->data;
		GaimStatus *account_status;
		
		if (!gaim_account_get_enabled(account, GAIM_GTK_UI))
			continue;
		
		if (gaim_account_is_disconnected(account))
			continue;
		
		account_status = gaim_account_get_active_status(account);
		
		if (gaim_account_is_connecting(account)) {
			tmpstatus = KIBA_STATUS_CONNECTING;
		} else if (gaim_status_is_online(account_status)) {
			if (!gaim_status_is_available(account_status)) {
				if (pending)
					tmpstatus = KIBA_STATUS_AWAY_PENDING;
				else
					tmpstatus = KIBA_STATUS_AWAY;
			}
			else {
				if (pending)
					tmpstatus = KIBA_STATUS_ONLINE_PENDING;
				else
					tmpstatus = KIBA_STATUS_ONLINE;
			}
		
			/* Check if any online accounts support chats */
			c = gaim_connections_get_all();
			while (c != NULL) {
				GaimConnection *gc = c->data;
				if (GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl)->chat_info != NULL) {
					online_account_supports_chat = TRUE;
					break;
				}
				c = c->next;
			}
		}
		else
			tmpstatus = KIBA_STATUS_OFFLINE;
		
		if (tmpstatus > newstatus)
		{
			const GaimStatusType *status_type = gaim_status_get_type (account_status);
			
			call_dbus_method (KIBA_DBUS_TITLE, (char*) gaim_status_type_get_name (status_type));
			newstatus = tmpstatus;
		}
	}
	
	if (status != newstatus) {
		status = newstatus;
		
		update_icon(status);
		
		if ((status == KIBA_STATUS_ONLINE_PENDING ||
				status == KIBA_STATUS_AWAY_PENDING) &&
				kiba_blinking_timer == 0) { /* if (blink) */
			kiba_blinking_timer = g_timeout_add(500, kiba_blink_icon, NULL);
		}
	}
	
	return FALSE;
}
	
/****************************
 * callbacks
 ****************************/

static void kiba_update_status_cb(void *data) {
	kiba_update_status();
}

static void kiba_conv_updated_cb(GaimConversation  *conv,
				 GaimConvUpdateType type) {
	if (type == GAIM_CONV_UPDATE_UNSEEN)
		kiba_update_status();
}

static void kiba_signon_cb(GaimConnection *gc) {
	kiba_update_status();
}

static void kiba_signoff_cb(GaimConnection *gc) {
	kiba_update_status();
}

static void buddy_signon_cb(GaimBuddy *buddy) {
}

static void buddy_signoff_cb(GaimBuddy *buddy) {
}

static void buddy_status_changed_cb(GaimBuddy  *buddy,
				    GaimStatus *old,
				    GaimStatus *new) {
}

static void kiba_icon_show_pref_changed_cb(const char   *name,
					   GaimPrefType  type,
					   gconstpointer value,
					   gpointer      data) {
	if (!strcmp(name, "/plugins/gtk/kiba/show_bubble_info")) {
		if (gaim_prefs_get_bool("/plugins/gtk/kiba/show_bubble_info"))
			kiba_update_status();
		else
			call_dbus_method (KIBA_DBUS_UNSET_BUBBLE, NULL);
	}
}

/****************************
 * Plugin stuff
 ****************************/

static gboolean plugin_load(GaimPlugin *plugin) {
	DBusGConnection *connection;
	DBusGProxy *proxy;
	GError *error = NULL;	

	void *blist_handle = gaim_blist_get_handle(); /* unused by gaim systray */
	void *conn_handle = gaim_connections_get_handle();
	void *conv_handle = gaim_conversations_get_handle();
	void *accounts_handle = gaim_accounts_get_handle();
	
	gaim_debug(GAIM_DEBUG_INFO, KIBA_PLUGIN_ID, "plugin loaded\n");
	
	g_type_init();
	
	error = NULL;
	connection = dbus_g_bus_get(DBUS_BUS_SESSION, &error);
	
	if (connection != NULL)
	{
		proxy = dbus_g_proxy_new_for_name(
				connection,
				KIBA_NAMESPACE,
				KIBA_DBUS_PATH,
				KIBA_NAMESPACE);
		
		call_dbus_method (KIBA_DBUS_NEW_ICON, "pidgin");
		
		call_dbus_method (KIBA_DBUS_TITLE, _("Offline"));
		call_dbus_method (KIBA_DBUS_SET_ICON, PATH_IMG_OFFLINE);
		
		listenKibaSignals(connection);
	}

	gaim_signal_connect(conn_handle, "signed-on", plugin,
			GAIM_CALLBACK(kiba_signon_cb), NULL);
	gaim_signal_connect(conn_handle, "signed-off", plugin,
			GAIM_CALLBACK(kiba_signoff_cb), NULL);

	gaim_signal_connect(accounts_handle, "account-status-changed", plugin,
			GAIM_CALLBACK(kiba_update_status_cb), NULL);
	gaim_signal_connect(accounts_handle, "account-connecting", plugin,
			GAIM_CALLBACK(kiba_update_status_cb), NULL);

	gaim_signal_connect(conv_handle, "received-im-msg", plugin,
			GAIM_CALLBACK(kiba_update_status_cb), NULL);
	gaim_signal_connect(conv_handle, "conversation-created", plugin,
			GAIM_CALLBACK(kiba_update_status_cb), NULL);
	gaim_signal_connect(conv_handle, "deleting-conversation", plugin,
			GAIM_CALLBACK(kiba_update_status_cb), NULL);
	gaim_signal_connect(conv_handle, "conversation-updated", plugin,
			GAIM_CALLBACK(kiba_conv_updated_cb), NULL);

	gaim_signal_connect(blist_handle, "buddy-signed-on", plugin,
			GAIM_CALLBACK(buddy_signon_cb), NULL);
	gaim_signal_connect(blist_handle, "buddy-signed-off", plugin,
			GAIM_CALLBACK(buddy_signoff_cb), NULL);
	gaim_signal_connect(blist_handle, "buddy-status-changed", plugin,
			GAIM_CALLBACK(buddy_status_changed_cb), NULL);
	
	kiba_update_status();
	
	return TRUE;
}

static gboolean plugin_unload(GaimPlugin *plugin) {
	call_dbus_method (KIBA_DBUS_REMOVE_ICON, NULL);
	
	return TRUE;
}	

static GtkWidget* plugin_config_frame(GaimPlugin *plugin) {
	GtkWidget *frame;
	GtkWidget *vbox, *hbox;
	GtkWidget *toggle;
	
	frame = gtk_vbox_new(FALSE, 18);
	gtk_container_set_border_width(GTK_CONTAINER(frame), 12);
	
	vbox = gaim_gtk_make_frame(frame, _("Kiba-Dock Icon Configuration"));
	hbox = gtk_hbox_new(FALSE, 18);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	
	toggle = gtk_check_button_new_with_mnemonic(_("Show Bubble info"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(toggle),
		 gaim_prefs_get_bool("/plugins/gtk/kiba/show_bubble_info"));
	g_signal_connect (G_OBJECT(toggle), "clicked", G_CALLBACK(kiba_icon_set_bool),
		(void *)"/plugins/gtk/kiba/show_bubble_info");
	gtk_box_pack_start(GTK_BOX(vbox), toggle, FALSE, FALSE, 0);
	
	gtk_widget_show_all(frame);
	return frame;
}

static GaimGtkPluginUiInfo ui_info = {
	plugin_config_frame
};

static GaimPluginInfo info = {
	GAIM_PLUGIN_MAGIC,		/* plugin magic */
	GAIM_MAJOR_VERSION,		/* major gaim version   */
	GAIM_MINOR_VERSION,		/* minor gaim version   */
	GAIM_PLUGIN_STANDARD,		/* type         */
	GAIM_GTK_PLUGIN_TYPE,		/* ui requirement   */
	0,				/* flags            */
	NULL,				/* dependencies */
	GAIM_PRIORITY_DEFAULT,		/* priority     */
	
	PLUGIN_NAME,			/* id           */
	PLUGIN_NAME,			/* name         */
	VERSION,			/* version      */
	NULL,				/* summary      */
	NULL,				/* description  */
	
	"Daniel Bessler <danielb@kiba-dock.org>",
					/* author       */
	WEBPAGE,			/* homepage     */

	plugin_load,			/* load         */
	plugin_unload,			/* unload       */
	NULL,				/* destroy      */
	
	&ui_info,			/* ui info      */
	NULL,				/* extra info       */
	NULL,				/* plugin prefs */
	NULL				/* plugin actions   */
};

static void init_plugin(GaimPlugin *plugin) {
	
#ifdef ENABLE_NLS
	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
#endif /* ENABLE_NLS */
	
	info.summary     = _("Kiba-Dock Pidgin Icon");
	info.description = _("A Plugin for Pidgin to show useful information in Kiba-Dock");
	
	gaim_prefs_add_none("/plugins/gtk/kiba");
	gaim_prefs_add_bool("/plugins/gtk/kiba/show_bubble_info", TRUE);
	gaim_prefs_connect_callback(plugin, "/plugins/gtk/kiba/show_bubble_info",
				    kiba_icon_show_pref_changed_cb, NULL);
}

GAIM_INIT_PLUGIN(hello_world, init_plugin, info);
